﻿using System;
using UnityEngine;
//using System.Collections;
//using System.Collections.Generic;
using SimpleJSON;

namespace MVRPlugin {
	public class SizeScript : MVRScript {

		public override void Init() {
			try {				
				if (containingAtom.type == "Person")
                {
					pluginLabelJSON.val = "Size Script (Person mode)";
					_person = containingAtom;
					JSONStorable rescaleObject = _person.GetStorableByID("rescaleObject");
					if (rescaleObject)
						_personScale = rescaleObject.GetFloatJSONParam("scale");
                }
				else
				{
					pluginLabelJSON.val = "Size Script";
				}
				
				JSONStorableFloat jspeed = new JSONStorableFloat("Speed", 1.0f, SetSpeedCallback, 0.01f, 5.0f, true);
				RegisterFloat(jspeed);
				speedSlider = CreateSlider(jspeed);
				
				UIDynamic spacer0 = CreateSpacer(false);
				spacer0.height = 30f;

				shrinkButton = CreateButton("Shrink");
				if (shrinkButton != null) {
					shrinkButton.button.onClick.AddListener(ShrinkCallback);
					JSONStorableAction ShrinkAction = new JSONStorableAction("Shrink", () =>{ShrinkCallback();});
					RegisterAction(ShrinkAction );

				}
				
				growButton = CreateButton("Grow");
				if (growButton != null) {
					growButton.button.onClick.AddListener(GrowCallback);
					JSONStorableAction GrowAction = new JSONStorableAction("Grow", () =>{GrowCallback();});
					RegisterAction(GrowAction );
				}
				
				toggleButton = CreateButton("Pause/Resume");
				if (toggleButton != null) {
					toggleButton.button.onClick.AddListener(ToggleCallback);
					JSONStorableAction PauseAction = new JSONStorableAction("Pause", () =>{ToggleCallback();});
					RegisterAction(PauseAction );
				}
				
				UIDynamic spacer1 = CreateSpacer(false);
				spacer1.height = 60f;
				
				resetButton = CreateButton("Reset size");
				if (resetButton != null) {
					resetButton.button.onClick.AddListener(ResetCallback);

				}
				
				JSONStorableFloat jrelativeSize = new JSONStorableFloat("Shrink/grow %", 100.0f, SetRelativeSizeCallback, 10.0f, 200.0f, true);
				RegisterFloat(jrelativeSize);
				relativeSizeSlider = CreateSlider(jrelativeSize, true);
				
				relativeSizeButton = CreateButton("Shrink/grow to %", true);
				if (relativeSizeButton != null) {
					relativeSizeButton.button.onClick.AddListener(FixedSizeCallback);
				}
				
				relativeSizeAction = new JSONStorableAction("SetNewSize", () =>{FixedSizeCallback();});
                RegisterAction(relativeSizeAction);
				
					
				if (_person)
				{
					UIDynamic spacer3 = CreateSpacer(true);
					spacer3.height = 100f;
				
					_syncPlayer = new JSONStorableBool("Sync with player", false);
					CreateToggle(_syncPlayer, true);
					
					JSONStorableFloat jframeDrop = new JSONStorableFloat("Frame drop rate", 2.0f, SetFrameDropCallback, 1.0f, 10.0f, true);
					RegisterFloat(jframeDrop);
					frameDropSlider = CreateSlider(jframeDrop, true);
				}
				
				_originalScale = _person ? _personScale.val : SuperController.singleton.worldScale;
				_targetScale = _minWorldScale;
				_shrink = true;
				_inProgress = false;
				_currentScale = _originalScale;
				SetScaledSpeed(1.0f);
				_relativeTargetSize = 1.0f;
				
				_frameDrop = 2;
				_frameDropCounter = 0;
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				if (_inProgress == true)
				{
					float scale = _currentScale;
					if (_shrink)
					{
						scale = Mathf.Max(_minWorldScale, scale - Time.deltaTime * _speed);
						if (scale <= Mathf.Max(_targetScale, _minWorldScale))
						{
							_targetScale = 0.0f;
							_inProgress = false;
						}
					}
					else
					{
						scale = Mathf.Min(_maxWorldScale, scale + Time.deltaTime * _speed);
						if (scale >= Mathf.Min(_targetScale, _maxWorldScale))
						{
							_targetScale = 0.0f;
							_inProgress = false;
						}
					}
					
					if (_person && (_frameDropCounter % _frameDrop == 0))
						_personScale.val = scale;
					
					if (!_person || _syncPlayer.val)
						SuperController.singleton.worldScale = scale;
					
					_currentScale = scale;
					_frameDropCounter++;
				}
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
		}


		// ***********************************************************************************
		// Code below is just for this example
		// ***********************************************************************************

		private Atom _person;
		private JSONStorableFloat _personScale;
		private JSONStorableBool _syncPlayer;
		
		protected UIDynamicButton shrinkButton;
		protected UIDynamicButton growButton;
		protected UIDynamicButton toggleButton;
		protected UIDynamicButton resetButton;
		protected UIDynamicSlider speedSlider;
		protected UIDynamicSlider frameDropSlider;
		
		protected JSONStorableAction relativeSizeAction;
		
		protected UIDynamicSlider relativeSizeSlider;
		protected UIDynamicButton relativeSizeButton;

		protected float _originalScale;
		protected float _userTargetScale;
		protected float _targetScale;
		protected float _speed;
		protected float _currentScale;
        protected bool _inProgress;
        protected bool _shrink;
		protected float _relativeTargetSize;
		
		private int _frameDrop;
        private int _frameDropCounter;
		
		protected const float _minWorldScale = 0.1f;
		protected const float _maxWorldScale = 10.0f;
		
		public void ShrinkCallback() {
			_targetScale = _minWorldScale;
			_shrink = true;
			_inProgress = true;
		}
		
		public void GrowCallback() {
			_targetScale = _maxWorldScale;
			_shrink = false;
			_inProgress = true;
		}
		
		public void ToggleCallback() {
			_inProgress = !_inProgress;
		}
		
		public void ResetCallback() {
			_targetScale = _originalScale;
			_shrink = (_currentScale > _originalScale) ? true : false;
			_inProgress = true;
		}
	
		public void SetUserSizeCallback() {
			_targetScale = _userTargetScale;
			_shrink = (_userTargetScale < 1.0f) ? true : false;
			_inProgress = true;
		}

		public void FixedSizeCallback() {
			_targetScale = _relativeTargetSize * _currentScale;
			_shrink = (_relativeTargetSize < 1.0f) ? true : false;
			_inProgress = true;
		}
		
		protected void SetSpeedCallback(JSONStorableFloat jf) {
			/*
				1.0 is default "average" speed
				shrinking for 1 second with 1.0 speed will cost you 10% size
				> 1.0 is faster
				< 1.0 is slower
			*/
			SetScaledSpeed(jf.val);
		}
		
		protected void SetFrameDropCallback(JSONStorableFloat jf) {
			_frameDrop = (int) Mathf.Floor(Mathf.Max(1.0f, jf.val));
		}
		
		protected void DummyCallback(JSONStorableFloat jf) {
		}
		
		protected void SetRelativeSizeCallback(JSONStorableFloat jf) {
			_relativeTargetSize = jf.val / 100.0f;
		}
		
		private void SetScaledSpeed(float value) {
			_speed = value * 0.05f;
		}
	}
}